'''
    (c) 2009 Clanbake
    Title: gg_give
    Version: 1.0.1
    Description:
        Fast (flexible) reload for gg5. Registers say, and client commands.
'''

# ==============================================================================
#   IMPORTS
# ==============================================================================
# Python imports

# EventScripts imports
import es
import time
import gamethread

# GunGame imports
import gungamelib

# ==============================================================================
#   ADDON REGISTRATION
# ==============================================================================
# Register this addon with EventScripts
info = es.AddonInfo()
info.name     = '!give (for GunGame5)'
info.version  = '1.0.1'
info.url      = 'http://www.seattlegungame.com'
info.basename = 'gungame/custom_addons/gg_give'
info.author   = 'Clanbake'

# ==============================================================================
#   GLOBALS
# ==============================================================================
# global variables

# User defined
giveCommand = gungamelib.getVariableValue('gg_give_command')
delayUsage = gungamelib.getVariableValue('gg_give_delay_usage')
limitPerRound = gungamelib.getVariableValue('gg_give_usage_per_round')
limitPerMap = gungamelib.getVariableValue('gg_give_usage_per_map')
weaponExceptions = gungamelib.getVariableValue('gg_give_weapon_exceptions')
showStatusHud = gungamelib.getVariableValue('gg_give_show_stats')
disableWarmupGive = gungamelib.getVariableValue('gg_give_disable_durring_warmup')
dispHudOnly = gungamelib.getVariableValue('gg_give_disp_msgs_in_hud')
# End user defined

if weaponExceptions:
    weaponExceptionsList = weaponExceptions.split(',')
else:
    weaponExceptionsList = 0

if limitPerRound >= limitPerMap and limitPerMap != 0:
    limitPerRound = 0

giveDisabled = False
weaponMapLimitReached = False
weaponRoundLimitReached = False
dictUsedPerMap = {}
dictUsedPerRound = {}
dictUsedLast = {}

# ==============================================================================
#   GAME EVENTS
# ==============================================================================
def load():           
    gg_give = gungamelib.registerAddon('gg_give')
    gg_give.setDisplayName('GG Give')
    gg_give.loadTranslationFile()
       
    # Set it up as a say command and a client command   
    if not es.exists('clientcommand', giveCommand):
        es.regclientcmd(giveCommand, 'gungame/custom_addons/gg_give/giveGun', 'Give Gun Command')

    if not es.exists('saycommand', giveCommand):
        es.regsaycmd(giveCommand, 'gungame/custom_addons/gg_give/giveGun', 'Give Gun Command')

def unload():
    gungamelib.unregisterAddon('gg_give')
    
    if es.exists('clientcommand', giveCommand):
        es.unregclientcmd(giveCommand)
        
    if es.exists('saycommand', giveCommand):
        es.unregsaycmd(giveCommand)

def round_start(ev):
    global weaponRoundLimitReached
    dictUsedPerRound.clear()
    dictUsedLast.clear()
    weaponRoundLimitReached = False    

def es_map_start(ev):
    global weaponRoundLimitReached
    global weaponMapLimitReached
    dictUsedPerRound.clear()
    dictUsedPerMap.clear()
    dictUsedLast.clear()
    weaponRoundLimitReached = False
    weaponMapLimitReached = False
    
# ==============================================================================
#   MAIN(SORT OF) FUNCTION
# ==============================================================================      
def giveGun():
    userid = es.getcmduserid()
    global weaponRoundLimitReached, weaponMapLimitReached
      
    if gungamelib.isDead(userid) or gungamelib.isSpectator(userid):
        return  
        
    if disableWarmupGive:
        if gungamelib.getGlobal('isWarmup') == 1:
            showSayMsg(userid, 'NotAllowedWarmup', {'command': giveCommand})
            return
    
    gungamePlayer = gungamelib.getPlayer(userid)
    weapon = gungamePlayer.getWeapon() 

    if weaponRoundLimitReached == False and weaponMapLimitReached == False:  
        if weapon in weaponExceptionsList:
            if dispHudOnly:
                showHudMsg(userid, 'NotAllowedHud', {'command': giveCommand, 'weapon': weapon})
            else:
                showSayMsg(userid, 'NotAllowed', {'command': giveCommand, 'weapon': weapon})
            
            return
 
        if delayUsage:
            if not giveGunTimer(userid):
                if dispHudOnly:
                    showHudMsg(userid, 'UsageDelayHud', {'command': giveCommand, 'delay': delayUsage})   
                else:
                    showSayMsg(userid, 'UsageDelay', {'command': giveCommand, 'delay': delayUsage})
                    
                return     
      
    if limitPerRound:
        if not getGiveLimit(userid,dictUsedPerRound,limitPerRound):
            giveStats(userid)
            weaponRoundLimitReached = True
            return
 
    if limitPerMap: 
        if not getGiveLimit(userid,dictUsedPerMap,limitPerMap):
            giveStats(userid)
            weaponMapLimitReached = True
            return      
        
    giveStats(userid) 
 
    gungamePlayer.stripPlayer()

    # Only delay if we are on linux
    if gungamelib.getOS() == 'posix':
        gamethread.delayed(0, delayedGiveNewWeapon, (userid))
    else:
        gungamePlayer.giveWeapon()

# ==============================================================================
#   HELPER FUNCTIONS
# ============================================================================== 
def delayedGiveNewWeapon(userid):
    if gungamelib.isDead(userid) or gungamelib.isSpectator(userid):
        return 
    
    gungamePlayer = gungamelib.getPlayer(userid)
    gungamePlayer.giveWeapon()

def getGiveLimit(userid,dictUsage,limitForType):
    if not userid in dictUsage:
        dictUsage[userid] = 1
        return 1
    if limitForType <= dictUsage[userid]:
        return 0    
    dictUsage[userid] += 1
    return 1

#TODO: fix - bit of kludge here  
def giveStats(userid):
    if showStatusHud and limitPerRound or showStatusHud and limitPerMap:
        round = -1
        map = -1
        if limitPerRound:
            if userid in dictUsedPerRound:
                round = limitPerRound - dictUsedPerRound[userid]
            else:
                round = limitPerRound
                
        if limitPerMap:  
            if userid in dictUsedPerMap:
                map = limitPerMap - dictUsedPerMap[userid]
            else:
                map = limitPerMap
  
        if round != -1 and map != -1 and round >= map:
            round = map
    
        if round >= 0:
            round = 'Round:%s/%s' % (round, limitPerRound)
        else:
            round = ''
            
        if map >= 0:
            map = 'Map:%s/%s' % (map, limitPerMap)
        else:
            map = ''

        showHudMsg(userid, 'HudStats', {'stats': round + ' ' + map})

def showHudMsg(userid, string, tokens):
    if gungamelib.canShowHints():
        gungamelib.hudhint('gg_give', userid, string, tokens) 

def showSayMsg(userid, string, tokens):
    gungamelib.msg('gg_give', userid, string, tokens)

def giveGunTimer(userid):
    if not userid in dictUsedLast:
        dictUsedLast[userid] = time.time()
        return 1
    if time.time() - dictUsedLast[userid] > delayUsage:
        dictUsedLast[userid] = time.time()
        return 1
    return 0
